//
// Copyright 2015-2021 by Garmin Ltd. or its subsidiaries.
// Subject to Garmin SDK License Agreement and Wearables
// Application Developer Agreement.
//

import Toybox.Application;
import Toybox.Application.Storage;
import Toybox.Lang;
import Toybox.Position;
import Toybox.Sensor;
import Toybox.Timer;
import Toybox.WatchUi;

//! This app demonstrates how to use a progress bar.
class AmbitApp extends Application.AppBase
{

private var wmm;

private var timer as Timer.Timer?;

//! Constructor
public function initialize()
{
    C.logger("app:onInitialize()");
    wmm = new WMM();
    C.mag_gps = Storage.getValue("mag_gps");
    AppBase.initialize();
}

//! Handle app startup
//! @param state Startup arguments
public function onStart(state as Dictionary?) as Void
{
    C.logger("app:onStart()");
    Position.enableLocationEvents(Position.LOCATION_CONTINUOUS, method(:onPosition));

    timer = new Timer.Timer();
    (timer as Timer.Timer).start(method(:timer_cb), 1000, true);
}

//! Handle app shutdown
//! @param state Shutdown arguments
public function onStop(state as Dictionary?) as Void
{
    C.logger("app:onStop()");
    Position.enableLocationEvents(Position.LOCATION_DISABLE, method(:onPosition));
}

//! Update the current position
//! @param info Position information
public function onPosition(info as Position.Info) as Void
{

    C.logger("app:onPosition()");
    C.new_gps(info, wmm);
}

//! Return the initial view for the app
//! @return Array Pair [View, InputDelegate]
public function getInitialView() as Array<Views or InputDelegates>?
{
    C.logger("app:getInitialView()");
    var first = C.METER_MAX - 1;
    if (WatchUi has :ViewLoop) {
        var factory = new PageIndicatorFactory();
        var viewLoop = new WatchUi.ViewLoop(factory, {:page => first, :wrap => true, :color => Graphics.COLOR_GREEN});
        return [viewLoop, new PageIndicatorDelegate(viewLoop)];
    } else {
        return [new AmbitView(first), new AmbitDelegate(first)] as Array<Views or InputDelegates>;
    }
}

// New app settings have been received so trigger a UI update
function onSettingsChanged() as Void {
    C.logger("onSettingsChanged()");
    C.settings();
    WatchUi.requestUpdate();
}

public function timer_cb() as Void
{

    C.gps_timer += 1;
    if (C.debug && (C.gps_timer > 3)) {
        C.new_gps(Position.getInfo(), wmm); }
    C.new_compass(Sensor.getInfo());
    WatchUi.requestUpdate();
}

}
