using Toybox.Graphics as Gfx;
using Toybox.Lang;
using Toybox.Math as Math;

using C as C;

class Barber
{

var nsegs;
var nstrip;

var fan;
var bbox;

var min_x;
var max_x;
var min_y;
var max_y;

var strip_w;
var strip_h;

public function initialize(min, max, width, orient, nsegs, nstrip)
{
    var p;

    me.nsegs = nsegs;
    me.nstrip = nstrip;
    fan = new [nsegs + nsegs + 2];
    width = (((Math.PI * 2) * width) / 360);
    var a = width / nsegs;
    var w2 = width/2;
    var delta = width/nsegs;
    var d = w2;
    for (var i = 0; i < nsegs; i += 1) {
        p = C.to_cart(min, d);
        fan[i] = p;
        d -= delta;
    }
    p = C.to_cart(min, -w2);
    fan[nsegs] = p;
    d = -w2;
    for (var i = 0; i < nsegs; i += 1) {
        p = C.to_cart(max, d);
        fan[nsegs + 1 + i] = p;
        d += delta;
    }
    fan[nsegs + nsegs + 1] = C.to_cart(max, w2);
    fan = C.translate(fan, C.cx, C.cy, orient + 90);
    p = C.to_cart(max, 0);
    var t = [ p ];
    t = C.translate(t, C.cx, C.cy, orient + 90);
    p = t[0];
    max_y = p[1];
    p = fan[0];
    min_y = p[1];
    p = fan[nsegs + 1];
    min_x = p[0];
    p = fan[nsegs + nsegs + 1];
    max_x = p[0];
    bbox = [ [ min_x, min_y ],
             [ min_x, max_y ],
             [ max_x, max_y ],
             [ max_x, min_y ] ];
    strip_h = max_y - min_y;
    strip_w = (max_x - min_x) / nstrip;
}

public function show(dc, value)
{
    var p1, p2;
    var strip;

    if (value > C.ceiling) {
        return; }

//    dc.setColor(Graphics.COLOR_WHITE, Graphics.COLOR_TRANSPARENT);
//    dc.fillPolygon(bbox);
//    dc.setColor(Graphics.COLOR_DK_GRAY, Graphics.COLOR_TRANSPARENT);
//    dc.fillPolygon(fan);

    dc.setColor(Graphics.COLOR_LT_GRAY, Graphics.COLOR_TRANSPARENT);

    var x2 = min_x;
    var x1 = min_x - (strip_h*0.8);
    var y2 = max_y;
    var y1 = max_y - strip_h;
    var w = strip_w / 3;
    while (x1 < max_x) {
        strip = [ [ x2, y2 ],
                  [ x1, y1 ],
                  [ x1 + w, y1 ],
                  [ x2 + w, y2 ] ];
        dc.fillPolygon(strip);
        x1 += strip_w;
        x2 += strip_w;
    }

    dc.setColor(Graphics.COLOR_BLACK, Graphics.COLOR_TRANSPARENT);
    var blank = [ [ min_x, max_y ],
                  fan[nsegs + 1] ];
    for (var i = nsegs; i >= 0; i -= 1) {
        blank.add(fan[i]); }
    blank.add(fan[nsegs + nsegs + 1]);
    blank.add([ max_x, max_y ]);
    blank.add([ x2, max_y ]);
    blank.add([ max_x + strip_h, min_y - 10] );
    blank.add([ min_x - strip_h, min_y - 10 ]);
    dc.fillPolygon(blank);

    blank = [ [ min_x - 10, max_y + 10 ] ];
    for (var i = (nsegs + 1); i <= (nsegs + nsegs + 1); i += 1) {
        blank.add(fan[i]); }
    blank.add([ max_x + 10, max_y + 10 ]);
    dc.fillPolygon(blank);

    dc.setColor(Graphics.COLOR_LT_GRAY, Graphics.COLOR_TRANSPARENT);
    dc.setPenWidth(1);
    for (var i = 0; i < (nsegs + nsegs + 1); i += 1) {
        p1 = fan[i];
        p2 = fan[i + 1];
        dc.drawLine(p1[0], p1[1], p2[0], p2[1]);
    }
    p1 = fan[0];
    p2 = fan[nsegs + nsegs + 1];
    dc.drawLine(p1[0], p1[1], p2[0], p2[1]);

    dc.setColor(Graphics.COLOR_WHITE, Graphics.COLOR_TRANSPARENT);
}

}
