using Toybox.Graphics as Gfx;
using Toybox.Math as Math;

using C as C;

class Meter
{

var title;
var max;
var tick;
var len;
var width;
var hands;
var color;

var sm_vert = new [4];
var bg_vert = new [4];

var bg_hand;
var sm_hand;
var ib_hand;

var barber;

public function initialize(title, max, tick, len, width, hands, color)
{

    me.title = title;
    me.max = max;
    me.tick = tick;
    me.len = len;
    me.width = width;
    me.hands = hands;
    me.color = color;

    var w2 = width / 2;

    //
    //  One small hash mark.  This is translated/rotated around
    //  the center of the screen to create all the hash marks.
    //
    sm_vert[0] = [ -w2, -max + len];
    sm_vert[1] = [ -w2, -max ];
    sm_vert[2] = [ +w2, -max ];
    sm_vert[3] = [ +w2, -max + len ];

    //
    //  One big hash mark.  This is translated/rotated around
    //  the center of the screen to create all the hash marks.
    //
    bg_vert[0] = [ -w2, -max + len + len];
    bg_vert[1] = [ -w2, -max ];
    bg_vert[2] = [ +w2, -max ];
    bg_vert[3] = [ +w2, -max + len + len ];

    var off = max * 0.25;
    bg_hand = new HandBg(max - len, off, C.WHITE);
    ib_hand = new HandIb(max, C.GREEN);
    if (hands > 2) {
        sm_hand = new HandSm(max * 0.75, off, C.WHITE);
        barber = new Barber(C.radius*0.4, C.radius*0.6, 40, 180, 8, 8);
    }
}

public function show(dc, value)
{
    var n, v, l, pos, th, font;

    var fhs = dc.getFontHeight(Graphics.FONT_SMALL);
    var fht = dc.getFontHeight(Graphics.FONT_XTINY);
    if (value == null) {
        dc.drawText(C.cx, C.cy - (fhs/2), Graphics.FONT_SMALL, "Wait GPS", Graphics.TEXT_JUSTIFY_CENTER);
        dc.drawText(C.cx, C.cy + (fht/2) + 4, Graphics.FONT_XTINY, C.gps_timer, Graphics.TEXT_JUSTIFY_CENTER);
        return;
    }

    //
    //  Unfortunately, we have to draw the barber pole first
    //    since it erases part of the meter face.  This wouldn't
    //    be an issue if we could set a polygon clipping region.
    //
    var vv = value[0].toNumber();
    if (hands > 2) {
        barber.show(dc, vv); }

    var p = new [1];
    p[0] = [ me.width/2, -me.max + me.len + me.len + 14 ];
    var d = -7.2;
    for (var i = 0; i < 100; i+= 2) {
        d += 7.2;
        if ((i % 10) == 0) {
            n = i / 10;
            pos = C.translate(p, C.cx, C.cy, d);
            l = pos[0];
            dc.drawText(l[0], l[1], Graphics.FONT_XTINY, n, Graphics.TEXT_JUSTIFY_CENTER |
                                                            Graphics.TEXT_JUSTIFY_VCENTER);
            v = C.translate(bg_vert, C.cx, C.cy, d);
        } else {
            v = C.translate(sm_vert, C.cx, C.cy, d);
        }
        dc.fillPolygon(v);
    }
    dc.setPenWidth(3);
    dc.drawCircle(C.cx, C.cy, C.radius);

    dc.drawText(C.cx, C.cy - (fhs/2), Graphics.FONT_SMALL, vv, Graphics.TEXT_JUSTIFY_CENTER);

    dc.drawText(C.cx, C.cy + (fht/2) + 4, Graphics.FONT_XTINY, value[1], Graphics.TEXT_JUSTIFY_CENTER);
    switch (hands) {

    case 2:
        bg_hand.show(dc, vv % 100, 100);
        ib_hand.show(dc, (vv / 10).toNumber(), 100);
        break;

    case 3:
        bg_hand.show(dc, (vv % 1000), 1000);
        sm_hand.show(dc, (vv % 10000), 10000);
        ib_hand.show(dc, (vv / 1000).toNumber(), 100);
        break;

    }
}

}
